rm(list=ls())
setwd("C:/your_working_directory/")
library(np)
options(np.messages=FALSE)
library(mvtnorm)

source("console.R")
source("util.R")
source("np.singleindex.ejl.R")

optim.method <- "BFGS"
optim.abstol <- 1e-4
optim.reltol <- 1e-4
nmulti <- 4

ns <- 1000
n <- c(250,500,1000)
r <- c(0,-1/2,-3/4)

beta.sim <- array(NA,c(ns,4,length(n),length(r)))
h.sim <- array(NA,c(ns,3,length(n),length(r)))

ptm <- proc.time()
for(js in 1:ns){
     set.seed(178+js)
     x1 <- rbeta(n[length(n)], shape1=2, shape2=2) -1/2

     for(jr in 1:length(r)){
     v <- rmvnorm(n[length(n)], mean = c(0,0), sigma = cbind(c(1,r[jr]),c(r[jr],1/8)), method="svd")
     x2 <- 2*dnorm(x1) + v[,1]
     y <- ifelse(x1 + x2 + - v[,2] > 0, 1, 0)

        for(jn in 1:length(n)){
        X <- cbind(x1[1:n[jn]], x2[1:n[jn]])
        bw.g <- npregbw(x2~x1,data=data.frame(x1=x1[1:n[jn]],x2=x2[1:n[jn]]),
                        bandwidth.compute=TRUE,residuals = TRUE)
        g <- npreg(bw.g)
        ghat <- residuals(g)
        g0 <- v[1:n[jn],1]

        ## Probit
        suppressWarnings(probit <- glm(y~-1+x1+x2,family=binomial(link = probit),data=data.frame(y=y[1:n[jn]],x1=x1[1:n[jn]],x2=x2[1:n[jn]])))
        beta.sim[js,1,jn,jr] <- coef(probit)[2]*sqrt(1/8)
        print(c(js,jr,jn,1))

        ## Ichimura
        bw.2 <- npindexbw(xdat=X, ydat=y[1:n[jn]], method="ichimura",
                optim.method =optim.method,
                optim.reltol=optim.reltol,
                optim.abstol=optim.abstol,
                nmulti=3)
        beta.sim[js,2,jn,jr] <- bw.2$beta[2]
        h.sim[js,1,jn,jr] <- bw.2$bw
        print(c(js,jr,jn,2))

        ## NEW Ichimura - TRUE
        bw.2$beta <- c(1,1)
        bw.3 <- np.singleindex.ejl(xdat=X, ydat=y[1:n[jn]],gdat=g0,wdat=1,bws=bw.2,
                                optim.method =optim.method,
                                optim.reltol=optim.reltol,
                                optim.abstol=optim.abstol,
                                nmulti=nmulti)
        beta.sim[js,3,jn,jr] <- bw.3$beta[2]
        h.sim[js,2,jn,jr] <- bw.3$bw
        print(c(js,jr,jn,3))

        ## NEW Ichimura - ESTIMATED
        bw.4 <- np.singleindex.ejl(xdat=X, ydat=y[1:n[jn]],gdat=ghat,wdat=1,bws=bw.2,
                                optim.method =optim.method,
                                optim.reltol=optim.reltol,
                                optim.abstol=optim.abstol,
                                nmulti=nmulti)
        beta.sim[js,4,jn,jr] <- bw.4$beta[2]
        h.sim[js,3,jn,jr] <- bw.4$bw
        print(c(js,jr,jn,4))
        print(beta.sim[js,,jn,jr])
        rm(X,g,ghat,g0,probit,bw.2,bw.3,bw.4)
        }
     rm(v,x2,y)
     }
rm(x1)
}
save(beta.sim,h.sim,file="sim.Rdata")
print(proc.time() - ptm)
